==> bettertrims-4.0.0%2B1.21.1-fabric-sources.jar: com/bawnorton/bettertrims/client/mixin/GuiGraphicsMixin.java <==
package com.bawnorton.bettertrims.client.mixin;

import com.bawnorton.bettertrims.client.tooltip.AbilityTooltipRenderer;
import com.llamalad7.mixinextras.expression.Definition;
import com.llamalad7.mixinextras.expression.Expression;
import com.llamalad7.mixinextras.injector.ModifyExpressionValue;
import com.llamalad7.mixinextras.injector.wrapmethod.WrapMethod;
import com.llamalad7.mixinextras.injector.wrapoperation.Operation;
import com.llamalad7.mixinextras.injector.wrapoperation.WrapOperation;
import dev.kikugie.fletching_table.annotation.MixinEnvironment;
import org.jetbrains.annotations.Nullable;
import org.joml.Vector2ic;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Unique;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfo;

import java.util.List;
import net.minecraft.class_1799;
import net.minecraft.class_327;
import net.minecraft.class_332;
import net.minecraft.class_5684;
import net.minecraft.class_768;
import net.minecraft.class_8000;

@MixinEnvironment("client")
@Mixin(class_332.class)
abstract class GuiGraphicsMixin {
	@Unique
	private static final ThreadLocal<class_768> TOOLTIP_DIMENSION_CAPTURE = new ThreadLocal<>();

	 //? if >=1.21.8 {
    /*@Definition(id = "deferredTooltip", field = "Lnet/minecraft/client/gui/GuiGraphics;deferredTooltip:Ljava/lang/Runnable;")
    @Expression("this.deferredTooltip = @(?)")
    @ModifyExpressionValue(
        method = "setTooltipForNextFrameInternal",
        at = @At("MIXINEXTRAS:EXPRESSION")
    )
    private Runnable renderTrimTooltip(
        Runnable original,
        Font font,
        List<ClientTooltipComponent> components,
        int x,
        int y,
        ClientTooltipPositioner positioner,
        @Nullable ResourceLocation background,
        boolean focused
    ) {
        ItemStack stack = AbilityTooltipRenderer.getStack();
        if (stack.isEmpty()) {
            AbilityTooltipRenderer.clearRendering();
            return original;
        }

        GuiGraphics self = (GuiGraphics) (Object) this;
        AbilityTooltipRenderer.clearStack();
        return () -> {
            original.run();
            AbilityTooltipRenderer.render(self, stack, font, TOOLTIP_DIMENSION_CAPTURE.get(), x, background);
            TOOLTIP_DIMENSION_CAPTURE.remove();
        };
    }
  *///?} else {
	@Inject(
			method = "renderTooltipInternal",
			at = @At("TAIL")
	)
	//?}
	private void renderTrimTooltip(class_327 font, List<class_5684> components, int mouseX, int mouseY, class_8000 tooltipPositioner, CallbackInfo ci) {
		class_1799 stack = AbilityTooltipRenderer.getStack();
		if (stack.method_7960()) {
			return;
		}

		class_332 self = (class_332) (Object) this;
		AbilityTooltipRenderer.clearStack();
		AbilityTooltipRenderer.render(self, stack, font, TOOLTIP_DIMENSION_CAPTURE.get(), mouseX, null);
		TOOLTIP_DIMENSION_CAPTURE.remove();
	}

	@WrapOperation(
			//? if >=1.21.8 {
			/*//? if fabric {
			method = "renderTooltip",
			//?} else {
			/^method = "renderTooltip(Lnet/minecraft/client/gui/Font;Ljava/util/List;IILnet/minecraft/client/gui/screens/inventory/tooltip/ClientTooltipPositioner;Lnet/minecraft/resources/ResourceLocation;Lnet/minecraft/world/item/ItemStack;)V",
			^///?}
			*///?} else {
			method = "renderTooltipInternal",
			//?}
			at = @At(
					value = "INVOKE",
					target = "Lnet/minecraft/client/gui/screens/inventory/tooltip/ClientTooltipPositioner;positionTooltip(IIIIII)Lorg/joml/Vector2ic;"
			)
	)
	private Vector2ic captureTooltipPosition(
			class_8000 instance,
			int screenWidth,
			int screenHeight,
			int mouseX,
			int mouseY,
			int tooltipWidth,
			int tooltipHeight,
			Operation<Vector2ic> original
	) {
		Vector2ic result = original.call(instance, screenWidth, screenHeight, mouseX, mouseY, tooltipWidth, tooltipHeight);
		TOOLTIP_DIMENSION_CAPTURE.set(new class_768(result.x(), result.y(), tooltipWidth, tooltipHeight));
		return result;
	}
}

==> bettertrims-4.0.2%2B1.21.10-fabric-sources.jar: com/bawnorton/bettertrims/client/mixin/GuiGraphicsMixin.java <==
package com.bawnorton.bettertrims.client.mixin;

import com.bawnorton.bettertrims.client.tooltip.AbilityTooltipRenderer;
import com.llamalad7.mixinextras.expression.Definition;
import com.llamalad7.mixinextras.expression.Expression;
import com.llamalad7.mixinextras.injector.ModifyExpressionValue;
import com.llamalad7.mixinextras.injector.wrapmethod.WrapMethod;
import com.llamalad7.mixinextras.injector.wrapoperation.Operation;
import com.llamalad7.mixinextras.injector.wrapoperation.WrapOperation;
import dev.kikugie.fletching_table.annotation.MixinEnvironment;
import org.jetbrains.annotations.Nullable;
import org.joml.Vector2ic;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Unique;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfo;

import java.util.List;
import net.minecraft.class_1799;
import net.minecraft.class_2960;
import net.minecraft.class_327;
import net.minecraft.class_332;
import net.minecraft.class_5684;
import net.minecraft.class_768;
import net.minecraft.class_8000;

@MixinEnvironment("client")
@Mixin(class_332.class)
abstract class GuiGraphicsMixin {
	@Unique
	private static final ThreadLocal<class_768> TOOLTIP_DIMENSION_CAPTURE = new ThreadLocal<>();

	//? if >=1.21.8 {
	@Definition(id = "deferredTooltip", field = "Lnet/minecraft/client/gui/GuiGraphics;deferredTooltip:Ljava/lang/Runnable;")
	@Expression("this.deferredTooltip = @(?)")
	@ModifyExpressionValue(
			method = "setTooltipForNextFrameInternal",
			at = @At("MIXINEXTRAS:EXPRESSION")
	)
	private Runnable renderTrimTooltip(
			Runnable original,
			class_327 font,
			List<class_5684> components,
			int x,
			int y,
			class_8000 positioner,
			@Nullable class_2960 background,
			boolean focused
	) {
		class_1799 stack = AbilityTooltipRenderer.getStack();
		if (stack.method_7960()) {
			AbilityTooltipRenderer.clearRendering();
			return original;
		}

		class_332 self = (class_332) (Object) this;
		AbilityTooltipRenderer.clearStack();
		return () -> {
			original.run();
			AbilityTooltipRenderer.render(self, stack, font, TOOLTIP_DIMENSION_CAPTURE.get(), x, background);
			TOOLTIP_DIMENSION_CAPTURE.remove();
		};
	}

	//?} else {
	/*@Inject(
			method = "renderTooltipInternal",
			at = @At("TAIL")
	)
	*///?}
	private void renderTrimTooltip(class_327 font, List<class_5684> components, int mouseX, int mouseY, class_8000 tooltipPositioner, CallbackInfo ci) {
		class_1799 stack = AbilityTooltipRenderer.getStack();
		if (stack.method_7960()) {
			return;
		}

		class_332 self = (class_332) (Object) this;
		AbilityTooltipRenderer.clearStack();
		AbilityTooltipRenderer.render(self, stack, font, TOOLTIP_DIMENSION_CAPTURE.get(), mouseX, null);
		TOOLTIP_DIMENSION_CAPTURE.remove();
	}

	@WrapOperation(
			//? if >=1.21.8 {
			//? if fabric {
			method = "renderTooltip",
			//?} else {
			/*method = "renderTooltip(Lnet/minecraft/client/gui/Font;Ljava/util/List;IILnet/minecraft/client/gui/screens/inventory/tooltip/ClientTooltipPositioner;Lnet/minecraft/resources/ResourceLocation;Lnet/minecraft/world/item/ItemStack;)V",
			*///?}
			//?} else {
			/*method = "renderTooltipInternal",
			*///?}
			at = @At(
					value = "INVOKE",
					target = "Lnet/minecraft/client/gui/screens/inventory/tooltip/ClientTooltipPositioner;positionTooltip(IIIIII)Lorg/joml/Vector2ic;"
			)
	)
	private Vector2ic captureTooltipPosition(
			class_8000 instance,
			int screenWidth,
			int screenHeight,
			int mouseX,
			int mouseY,
			int tooltipWidth,
			int tooltipHeight,
			Operation<Vector2ic> original
	) {
		Vector2ic result = original.call(instance, screenWidth, screenHeight, mouseX, mouseY, tooltipWidth, tooltipHeight);
		TOOLTIP_DIMENSION_CAPTURE.set(new class_768(result.x(), result.y(), tooltipWidth, tooltipHeight));
		return result;
	}
}
